#! /usr/bin/python

"""//Web service to get the number of credits in a batch order//
 
- File: get_credits.py

- Arguments - Using the HTTP GET method:
 - batch_id -- The batch identifier, as a string
 - > If some arguments are missing, a web form will be displayed.
 
- Displays a Web page containing:
 - ``<div class="qos">N</div>`` where N is the number of credit in the order of the given batch_id 

- Notes: 
 - A database configuration file named "db.cfg" must be present in the directory.

"""

import sys
import os
sys.path.append(os.path.join(sys.path[0],".."))
os.chdir(sys.path[0])
from tools import cgi_func
from tools import db_func
from tools import log_func
log=log_func.get_spequlos_log(__file__)



def get_credits(conn,batch_id):
    """Function implementing the get_credits operation

    Arguments:
    conn -- The CreditSystem's DB connection, as a MySQL.connections.Connection object
    batch_id -- The batch identifier, as a string
    
    The function returns the number of credits in the order of batch identified by batch_id, as an integer.

    """
    if not isinstance(conn,db_func.db_connection) or conn.open==0:
        log.error("The conn argument must be an open connection to the database !")
        return None         
    if not isinstance(batch_id,str):
        log.error("The batch_id argument must be a string !")
        return None
    
    c=conn.cursor()

    log.info("Retrieving order for batch identifier %s " % batch_id)
    try:
        c.execute("SELECT credit_balance FROM QoSOrder WHERE batch_id='%s'" % batch_id)
    except db_func.DBError as e:
        log.error("Error during the SQL command execution !")
        log.error(e)
        return None     
    res = c.fetchall()
    
    if len(res)==0:
        log.warning("No order associated to this batch id")
        return 0
    elif len(res)>1:
        log.error("Several orders associated with batch identifier %d in the database ! 'batch_id' should be QoSOrder PRIMARY KEY !" % batch_id)
        return None
    else :
        try:
            order_credit = int(res[0][0])
            if order_credit<0:
                raise ValueError("Negative credit_balance")
        except ValueError as e:
            log.error("The 'credit_balance' in QoSOrder table must be a positive or zero integer !")
            log.error(e)
            return None
        else:
            return order_credit
        
if __name__=="__main__":
    cgi_func.print_header()

    if cgi_func.get_form_value('batch_id') is None:
        cgi_func.print_form(['batch_id'])

    else:
        batch_id = cgi_func.get_form_value('batch_id')
        conn=db_func.connect(db_func.read_db_config("db.cfg"))
        cgi_func.print_result( get_credits(conn,batch_id))
        db_func.disconnect(conn)
        
    cgi_func.print_footer()


