#! /usr/bin/python

"""//Functions used by the scheduler related to Cloud worker management//

- File: cloudworker.py

"""

import sys
import os
sys.path.append(os.path.join(sys.path[0],".."))
os.chdir(sys.path[0])
from tools import web_func
from tools import conf_func
from tools import log_func
log=log_func.get_spequlos_log(__file__)


def start_cloud_worker(cloud_id,cloud_config_file,dg_id,dg_config_file,batch_id,batch_data,instance_id,handler):
    """//Start a cloud worker//

    - > According to the way CloudWorker is handled (libcloud or command), this function load the appropriate implementation to start the CloudWorker. 
     
    - Arguments:
     - cloud_id -- The Cloud service identifier
     - cloud_config_file -- The Cloud service configuration file
     - dg_id -- The supported Desktop Grid identifer
     - dg_config_file -- The Desktop Grid configuration file
     - batch_id -- The supported batch_id identifier
     - batch_data -- Data related to the supported batch. For XWHEP, it stores the batch UID.     
     - instance_id -- The identifier of the Cloud instance to start
     - handler -- The handler used to start Cloud worker. "libcloud" or "command" are supported.
    
    - The function return True or some Cloud instance data specific to the Cloud service if the Cloud worker is correctly started, or False otherwise. For libcloud handler it is the started node id.
    
    """
    
    if handler=="libcloud":
        dg_params = conf_func.read_config(dg_config_file)
        if not "CW_SSH_CMD" in dg_params:
            log.error("No CW_SSH_CMD parameter in DG configuration file %s" % dg_config_file)
            raise Exception("No CW_SSH_CMD parameter in %s" % dg_config_file)
        env_export="export SQS_BATCH_ID=%s; export SQS_BATCH_DATA=%s; " % (batch_id, batch_data)
        list_of_commands = [env_export+dg_params["CW_SSH_CMD"]]
        
        from libcloud_cloudworker import Libcloud_CW_Starter as CW_Starter
        cw = CW_Starter(cloud_config_file,list_of_commands,instance_id)
        
    elif handler=="command":
        from command_cloudworker import Command_CW_Starter as CW_Starter
        cw = CW_Starter(cloud_id,cloud_config_file,dg_id,dg_config_file,batch_id,batch_data,instance_id)

    else:
        log.error("No implementation for %s handler" % handler)
        raise Exception("No handler %s" % handler)  

    cw.start()
    cw.join()
    if cw.execution_is_success:
        log.info("CloudWorker is started")
        return cw.specific_data or True
    else:
        log.error("CloudWorker starting failed")
        return False



def stop_cloud_worker(cloud_id,cloud_config_file,dg_id,dg_config_file,batch_id,batch_data,instance_id,instance_data,handler):
    """//Stop a cloud worker//

    - > According to the way CloudWorker is handled (libcloud or command), this function load the appropriate implementation to stop the CloudWorker. 
     
    - Arguments:
     - cloud_id -- The Cloud service identifier
     - cloud_config_file -- The Cloud service configuration file
     - dg_id -- The supported Desktop Grid identifer
     - dg_config_file -- The Desktop Grid configuration file
     - batch_id -- The supported batch_id identifier
     - batch_data -- Data related to the supported batch. For XWHEP, it stores the batch UID.          
     - instance_id -- The identifier of the Cloud instance to start
     - instance_data -- Some data specific to the Cloud service and to the instance to stop (output of start_cloud_worker). For libcloud handler it is the node id to stop.
     - handler -- The handler used to start Cloud worker. "libcloud" or "command" are supported.
              
    - The function return True if the Cloud worker is correctly stopped, False otherwise.
    
    """
    
    if handler=="libcloud":       
        from libcloud_cloudworker import Libcloud_CW_Stopper as CW_Stopper
        cw = CW_Stopper(cloud_config_file,instance_data)
        
    elif handler=="command":
        from command_cloudworker import Command_CW_Stopper as CW_Stopper
        cw = CW_Stopper(cloud_id,cloud_config_file,dg_id,dg_config_file,batch_id,batch_data,instance_id,instance_data)

    else:
        log.error("No implementation for %s handler" % handler)
        raise Exception("No implementation %s handler" % handler)
  
    cw.start()
    cw.join()
    if cw.stop_is_success:
        log.info("CloudWorker is stopped")
        return True
    else:
        log.error("CloudWorker stop failed")
        return False




if __name__=="__main__":
    import sys
    handler=sys.argv[1]
    
    start_ok = start_cloud_worker("cloud_none",sys.argv[2],"dg_none",sys.argv[3],"batch_none","instance_none",handler)

    if start_ok:
        print "Press enter to stop the cloud worker"
        x=raw_input()
        stop_ok = stop_cloud_worker("cloud_none",sys.argv[2],"dg_none",sys.argv[3],"batch_none","instance_none",start_ok)
        if stop_ok:
            print "Cloud worker correctly stopped"

