#! /usr/bin/python

"""//Scheduler implementation to remotely call CreditSystem functions//

- File: creditsystem_rpc.py

"""

import sys
import os
sys.path.append(os.path.join(sys.path[0],".."))
os.chdir(sys.path[0])
from tools import web_func
from tools import conf_func
from tools import log_func
log=log_func.get_spequlos_log(__file__)




def get_credits(batch_id):
    """//Remote call of Credit System's get_credits//
    
    - > The function calls and retrieves the result of the get_credits function from the Credit System web server.
    It needs a file named "creditsystem.cfg" containing the line "address=<url>", with <url> the url to access to the QoS Info web server, to be present in the directory. 
    
    - Arguments:
     - batch_id -- The batch identifier, as a string
    
    - The function returns the number of credits in the batch order, as an integer
    
    """  
    if not isinstance(batch_id,str):
        log.error("get_credits() batch_id argument must be a string")
        return None    
    
    params=conf_func.read_config("creditsystem.cfg")
    
    url="http://"+params['address']+"/get_credits.py?batch_id="+batch_id

    res=web_func.call(url)
                                                    
    if len(res)!=1:
        log.error("There should be 1 result from url %s" % url)
        return None
    try:
        r=int(res[0])
        if r<0:
            raise ValueError("Negative number of credit")
    except ValueError as e:
        log.error("get_credits result must be a positive integer")
        log.error(e)
        return None
    else:
        return r            


def pay(batch_id):
    """//Remote call of Credit System's pay//
    
    - > The function calls the pay function from the Credit System web server.
    It needs a file named "creditsystem.cfg" containing the line "address=<url>", with <url> the url to access to the QoS Info web server, to be present in the directory. 
    
    - Arguments:
     - batch_id -- The batch identifier, as a string
       
    """  
    if not isinstance(batch_id,str):
        log.error("pay() batch_id argument must be a string")
        return None    
            
    params=conf_func.read_config("creditsystem.cfg")
    
    url="http://"+params['address']+"/pay.py?batch_id="+batch_id

    res=web_func.call(url)
                                                    
    if len(res)!=1:
        log.error("There should be 1 result from url %s" % url)
        return None
    if res[0]!="True":
        log.error("Result from url %s should be True" % url)
        return None
    return True



def bill(batch_id,credit):
    """//Remote call of Credit System's bill//
    
    - > The function calls and retrieves the result of the pay function from the Credit System web server.
    It needs a file named "creditsystem.cfg" containing the line "address=<url>", with <url> the url to access to the QoS Info web server, to be present in the directory. 
    
    - Arguments:
     - batch_id -- The batch identifier, as a string
     - credit -- The number of credit to bill (spent by cloud ressources)
       
    - The function returns True if there is enough credits for this bill, False otherwise
           
    """  
    if not isinstance(batch_id,str):
        log.error("bill() batch_id argument must be a string")
        return None
    if not isinstance(credit,int) or credit<0:
        log.error("bill() credit argument must be a positive integer")
        return None
                    
    params=conf_func.read_config("creditsystem.cfg")
    
    url="http://"+params['address']+"/bill.py?batch_id="+batch_id+"&credit="+str(credit)

    res=web_func.call(url)
                                                    
    if len(res)!=1:
        log.error("Several or no results from url %s" % url)
        return None
    if not (res[0]=="True" or res[0]=="False"):
        log.error("Result from url %s should be True or False" % url)
        return None
    if res[0]=="True": 
        return True
    else:
        return False

