#! /usr/bin/python

"""//Web service to enable QoS for a batch//
 
- > The start_qos function is the first to be called, because this function will return the batch identifier that must be used for the other QoS related operations
 
- File: start_qos.py

- Arguments - Using the HTTP GET method:
 - dg_identifier -- The desktop grid identifier where batch will be executed, as a string
 - > If some arguments are missing, a web form will be displayed.
 
- Displays a Web page containing:
 - ``<div class="qos">batch_id</div>`` with batch_id the batch identifier to use in future QoS operations
 
- Notes: 
 - A database configuration file named "db.cfg" must be present in the directory.

"""
import sys
import os
sys.path.append(os.path.join(sys.path[0],".."))
os.chdir(sys.path[0])
from tools import db_func
from tools import conf_func
from tools import cgi_func
from tools import log_func
log=log_func.get_spequlos_log(__file__)

import dg_rpc

import uuid


def start_qos(conn,dg_name):
    """//Function implementing the start_qos operation//

    - > The function perform the necessary operations (DB insertions) to enable future managment of the batch

    - Arguments:
     - db -- The scheduler's DB connection, as a SpeQuloS_DB object
     - dg_id -- The desktop grid identifier where batch is executed, as a string
    
    - The function returns the batch identifier, as a string
    """    
        
    log.info("Checking if this desktop grid is registered")
    dgs = db.sql_select("DesktopGrid",('dg_id','dg_conf_file'),"dg_id='%s'" % dg_id)
    
    if len(dgs)==0:
        log.error("Unable to find the desktop grid, is it registered ?")
        return False
    elif len(dgs)>1:
        log.error("Database error: we should have one result! 'dg_id' must be the PRIMARY KEY of DesktopGrid table")
        return False   
    dg=dgs[0]
    
    #For BOINC we need a small integer for the batch_id to be conform with 'batch' field in BOINC database
    batch_id="%d" % int( uuid.uuid4().int % 0x3FF )

    log.info("Preconfiguring the DG with the batch information...")
    res=dg_rpc.preconfigure_QoS(batch_id,dg['dg_id'],dg['dg_conf_file']) 
    if res == None:
        log.error("Error during preconfigure_QoS with %s, %s, %s" % (batch_id,dg['dg_id'],dg['dg_conf_file']))
        batch_data=''
    elif res == True:    
        log.info("Done, no specific batch_data")
        batch_data=''
    else:
        log.info("Done, and we have data to insert in the batch_data field of the database")
        batch_data=res

    log.info("Batch_id to use : %s . Inserting QoSBatch entry in the DB..." % batch_id)
    db.sql_insert("QoSBatch",('batch_id','dg_id','batch_data'),(batch_id,dg['dg_id'],batch_data))

    #For XWHEP, we need to submit BoT using the group UUID, contained in the last part of the batch_data 
    log.info("Reading DG configuration file")      
    dg_params = conf_func.read_config(dg['dg_conf_file'])
    if not "DG_TYPE" in dg_params:
        log.error("No DG_TYPE parameter in DG configuration file %s" % dg['dg_conf_file'])
        raise Exception("No DG_TYPE parameter in %s" % dg['dg_conf_file'])
    
    if dg_params["DG_TYPE"]=="XWHEP":
        group_uid=batch_data.split("/")[-1]
        cgi_func.print_message("The following UID must be used for XWHEP submission: %s" % group_uid)

    log.info("start_qos() completed")
    return batch_id


if __name__=="__main__":
    cgi_func.print_header()

    if (cgi_func.get_form_value('dg_identifier') is None):
        cgi_func.print_message("Please provide the identifier of the Desktop Grid you want to use")
        cgi_func.print_form(['dg_identifier'])

    else:
        dg_id = cgi_func.get_form_value('dg_identifier')
        db = db_func.SpeQuloS_DB(os.path.join(sys.path[0],"db.cfg"))                
    
        batch_id = start_qos(db,dg_id)
        if batch_id is None:
            cgi_func.print_message("<a href='start_qos.py'>Go back</a>")
        else:
            cgi_func.print_message("Use this identifier for your batch:")
            cgi_func.print_result(batch_id)
            
    cgi_func.print_footer()

