#!/usr/bin/python

"""//Module for logging//

- File: log_func.py

"""
import logging
import sys

def get_spequlos_log(name=None,html=True):
    """//Creates and configure logger for SpeQuloS logging//

    - Arguments:
     - name --- The logger name as a string, should be the name of the module using the logger
     - html --- If True (default), will print HTML log. Will print plain text otherwise

    - The function returns a python logging.Logger instance

    """
    qoslog = logging.getLogger(name)
    qoslog.setLevel(logging.DEBUG)

    # Handler could be used to store logs in rotating files 
    #ch = logging.handlers.RotatingFileHandler(LOG_FILENAME, maxBytes=1024, backupCount=100)

    # Handler can also display log on standard output
    ch = logging.StreamHandler(sys.stdout)

    if html:    
        # Create a formatter suitable for displaying log on web pages
        formatter = logging.Formatter("<p>%(asctime)s - %(name)s - %(levelname)s - %(message)s</p>")
    else:
        formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
    
    # Add formatter to handler
    ch.setFormatter(formatter)

    # Add handler to logger
    qoslog.addHandler(ch)

    qoslog.info("QoS logging system started for %s" % name)
    return qoslog
